@@ -9,11 +9,10 @@
 #
 import re
 import math
-import locale as _locale
 from calendar import isleap, leapdays
 from decimal import Decimal
 from operator import attrgetter
-from typing import Optional, Union, SupportsFloat
+from typing import Any, Iterator, List, Match, Optional, Union, SupportsFloat
 
 ###
 # Common sets constants
@@ -169,6 +168,20 @@ def ordinal(n: int) -> str:
         return '%dth' % n
 
 
+def get_double(value: Union[SupportsFloat, str], xsd_version: str = '1.0') -> float:
+    if isinstance(value, str):
+        value = collapse_white_spaces(value)
+        if value in NUMERIC_INF_OR_NAN or xsd_version != '1.0' and value == '+INF':
+            if value == 'NaN':
+                return math.nan  # for NaN use the predefined instance to keep identity
+        elif value.lower() in INVALID_NUMERIC:
+            raise ValueError(f'invalid value {value!r} for xs:double/xs:float')
+    elif math.isnan(value):
+        return math.nan
+
+    return float(value)
+
+
 def numeric_equal(op1: SupportsFloat, op2: SupportsFloat) -> bool:
     if op1 == op2:
         return True
@@ -181,6 +194,22 @@ def numeric_not_equal(op1: SupportsFloat, op2: SupportsFloat) -> bool:
     return not math.isclose(op1, op2, rel_tol=1e-7, abs_tol=0.0)
 
 
+def is_nan(x: Any) -> bool:
+    return isinstance(x, float) and math.isnan(x)
+
+
+def equal(op1: Any, op2: Any) -> bool:
+    if isinstance(op1, float) and math.isnan(op1):
+        return isinstance(op2, float) and math.isnan(op2)
+    return bool(op1 == op2)
+
+
+def not_equal(op1: Any, op2: Any) -> bool:
+    if isinstance(op1, float) and math.isnan(op1):
+        return not isinstance(op2, float) or not math.isnan(op2)
+    return bool(op1 != op2)
+
+
 def match_wildcard(name: str, wildcard: str) -> bool:
     if wildcard == '*' or wildcard == '*:*':
         return True
@@ -195,16 +224,63 @@ def match_wildcard(name: str, wildcard: str) -> bool:
         return False
 
 
-def get_locale_category(category: int) -> str:
-    """
-    Gets the current value of a locale category. A replacement
-    of locale.getdefaultlocale(), deprecated since Python 3.11.
-    """
-    locale = _locale.setlocale(category, None)
-    if locale == 'C':
-        # locale category does not seem to be configured, so get the user
-        # preferred locale and then restore the  previous state
-        locale = _locale.setlocale(category, '')
-        _locale.setlocale(category, 'C')
+def escape_json_string(s: str, escaped: bool = False) -> str:
+    if escaped:
+        s = s.replace('\\"', '"')
+    else:
+        s = s.replace('\\', '\\\\')
 
-    return locale
+    s = s.replace('\"', '\\"').\
+        replace('\b', r'\b').\
+        replace('\r', r'\r').\
+        replace('\n', r'\n').\
+        replace('\t', r'\t').\
+        replace('\f', r'\f').\
+        replace('/', r'\/')
+    return ''.join(
+        rf'\u{ord(x):04X}' if 1 <= ord(x) <= 31 or 127 <= ord(x) <= 159 else x
+        for x in s
+    )
+
+
+def unescape_json_string(s: str) -> str:
+
+    def unicode_escape_callback(match: Match[str]) -> str:
+        return chr(int(match.group(1).upper(), 16))
+
+    s = s.replace('\\"', '\"').\
+        replace(r'\b', '\b').\
+        replace(r'\r', '\r').\
+        replace(r'\n', '\n').\
+        replace(r'\t', '\t').\
+        replace(r'\f', '\f').\
+        replace(r'\/', '/').\
+        replace('\\\\', '\\')
+
+    return re.sub(r'\\u([0-9A-Fa-f]{4})', unicode_escape_callback, s)
+
+
+def iter_sequence(obj: Any) -> Iterator[Any]:
+    if obj is None:
+        return
+    elif isinstance(obj, list):
+        for item in obj:
+            yield from iter_sequence(item)
+    else:
+        yield obj
+
+
+def split_function_test(function_test: str) -> List[str]:
+    if not function_test.startswith('function('):
+        return []
+    elif function_test == 'function(*)':
+        return ['*']
+
+    parts = function_test[9:].partition(') as ')
+    if parts[0]:
+        sequence_types = parts[0].split(', ')
+        sequence_types.append(parts[2])
+    else:
+        sequence_types = [parts[2]]
+
+    return sequence_types
