@@ -16,15 +16,17 @@ import operator
 from copy import copy
 from decimal import Decimal, DivisionByZero
 
-from ..exceptions import ElementPathError, ElementPathTypeError
-from ..helpers import OCCURRENCE_INDICATORS, numeric_equal, numeric_not_equal
+from ..exceptions import ElementPathError
+from ..helpers import OCCURRENCE_INDICATORS, numeric_equal, numeric_not_equal, \
+    node_position, get_double
 from ..namespaces import XSD_NAMESPACE, XSD_NOTATION, XSD_ANY_ATOMIC_TYPE, \
-    XSI_NIL, get_namespace, get_expanded_name
-from ..datatypes import UntypedAtomic, QName, AnyURI, Duration, Integer, DoubleProxy10
-from ..xpath_nodes import TypedElement, is_xpath_node, \
-    match_attribute_node, is_element_node, is_document_node
+    XSD_UNTYPED, get_namespace, get_expanded_name
+from ..datatypes import get_atomic_value, UntypedAtomic, QName, AnyURI, \
+    Duration, Integer, DoubleProxy10
+from ..xpath_nodes import ElementNode, DocumentNode, XPathNode, AttributeNode
+from ..sequence_types import is_instance
 from ..xpath_context import XPathSchemaContext
-from ..xpath_token import XPathFunction
+from ..xpath_tokens import XPathFunction
 
 from .xpath2_parser import XPath2Parser
 
@@ -46,7 +48,7 @@ def nud_as_and_of_symbols(self):
 # Variables
 @method('$', bp=90)
 def nud_variable_reference(self):
-    self.parser.expected_name('(name)', 'Q{')
+    self.parser.expected_next('(name)', 'Q{')
     self[:] = self.parser.expression(rbp=90),
     return self
 
@@ -63,22 +65,34 @@ def evaluate_variable_reference(self, context=None):
 
     varname = self[0].value
     try:
-        return context.variables[varname]
+        value = context.variables[varname]
     except KeyError:
-        if isinstance(context, XPathSchemaContext):
-            try:
-                sequence_type = self.parser.variable_types[varname].strip()
-            except KeyError:
-                pass
-            else:
-                if sequence_type[-1] in OCCURRENCE_INDICATORS:
-                    sequence_type = sequence_type[:-1]
+        pass
+    else:
+        return value if value is not None else []
 
-                if QName.pattern.match(sequence_type) is not None:
-                    return self.parser.get_atomic_value(sequence_type)
-                return UntypedAtomic('')
+    if isinstance(context, XPathSchemaContext):
+        try:
+            sequence_type = self.parser.variable_types[varname].strip()
+        except KeyError:
+            return []
+        else:
+            if sequence_type[-1] in OCCURRENCE_INDICATORS:
+                sequence_type = sequence_type[:-1]
 
-    raise self.missing_name('unknown variable %r' % str(varname))
+            if QName.pattern.match(sequence_type) is not None:
+                try:
+                    type_name = get_expanded_name(sequence_type, self.parser.namespaces)
+                except KeyError:
+                    pass
+                else:
+                    xsd_type = context.root.elem.xpath_proxy.get_type(type_name)
+                    if xsd_type is not None:
+                        return get_atomic_value(xsd_type)
+
+            return UntypedAtomic('1')
+
+    raise self.error('XPST0008', 'unknown variable %r' % str(varname))
 
 
 ###
@@ -93,13 +107,14 @@ def select_intersect_and_except_operators(self, context=None):
         raise self.missing_context()
 
     s1, s2 = set(self[0].select(copy(context))), set(self[1].select(copy(context)))
-    if any(not is_xpath_node(x) for x in s1) or any(not is_xpath_node(x) for x in s2):
+    if any(not isinstance(x, XPathNode) for x in s1) \
+            or any(not isinstance(x, XPathNode) for x in s2):
         raise self.error('XPTY0004', 'only XPath nodes are allowed')
 
     if self.symbol == 'except':
-        yield from context.iter_results(s1 - s2)
+        yield from sorted(s1 - s2, key=node_position)
     else:
-        yield from context.iter_results(s1 & s2)
+        yield from sorted(s1 & s2, key=node_position)
 
 
 ###
@@ -107,8 +122,7 @@ def select_intersect_and_except_operators(self, context=None):
 @method('if', bp=20)
 def nud_if_expression(self):
     if self.parser.next_token.symbol != '(':
-        token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
-        return token.nud()
+        return self.as_name()
 
     self.parser.advance('(')
     self[:] = self.parser.expression(5),
@@ -143,8 +157,7 @@ def select_if_expression(self, context=None):
 def nud_quantified_expressions(self):
     del self[:]
     if self.parser.next_token.symbol != '$':
-        token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
-        return token.nud()
+        return self.as_name()
 
     while True:
         self.parser.next_token.expected('$')
@@ -194,8 +207,7 @@ def evaluate_quantified_expressions(self, context=None):
 def nud_for_expression(self):
     del self[:]
     if self.parser.next_token.symbol != '$':
-        token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
-        return token.nud()
+        return self.as_name()
 
     while True:
         self.parser.next_token.expected('$')
@@ -237,28 +249,13 @@ def select_for_expression(self, context=None):
 @method('treat', bp=61)
 def led_sequence_type_based_expressions(self, left):
     self.parser.advance('of' if self.symbol == 'instance' else 'as')
-    if self.parser.next_token.label not in ('kind test', 'sequence type', 'function test'):
-        self.parser.expected_name('(name)', ':')
-
-    try:
-        self[:] = left, self.parser.expression(rbp=self.rbp)
-    except ElementPathTypeError as err:
-        message = getattr(err, 'message', str(err))
-        raise self.error('XPST0003', message) from None
-
-    next_symbol = self.parser.next_token.symbol
-    if self[1].symbol != 'empty-sequence' and next_symbol in ('?', '*', '+'):
-        self[2:] = self.parser.symbol_table[next_symbol](self.parser),  # Add nullary token
-        self.parser.advance()
+    self[:] = left, self.parse_sequence_type()
     return self
 
 
 @method('instance')
 def evaluate_instance_expression(self, context=None):
-    if len(self) > 2:
-        occurs = self[2].symbol
-    else:
-        occurs = self[1].occurrence
+    occurs = self[1].occurrence
     position = None
 
     if self[1].symbol == 'empty-sequence':
@@ -269,27 +266,33 @@ def evaluate_instance_expression(self, context=None):
         if context is None:
             raise self.missing_context()
 
+        context = copy(context)
+
         for position, context.item in enumerate(self[0].select(context)):
+            if context.axis is None:
+                context.axis = 'self'
+
             result = self[1].evaluate(context)
-            if result is None or isinstance(result, list) and not result:
+            if isinstance(result, list) and not result:
                 return occurs in ('*', '?')
             elif position and (occurs is None or occurs == '?'):
                 return False
         else:
             return position is not None or occurs in ('*', '?')
     else:
+        type_name = self[1].source.rstrip('*+?')
         try:
-            qname = get_expanded_name(self[1].source, self.parser.namespaces)
+            qname = get_expanded_name(type_name, self.parser.namespaces)
         except KeyError as err:
             raise self.error('XPST0081', "namespace prefix {} not found".format(err))
 
         for position, item in enumerate(self[0].select(context)):
             try:
-                if not self.parser.is_instance(item, qname):
+                if not is_instance(item, qname, self.parser):
                     return False
             except KeyError:
-                msg = "atomic type %r not found in in-scope schema types"
-                raise self.error('XPST0051', msg % self[1].source) from None
+                msg = f"atomic type {type_name!r} not found in in-scope schema types"
+                raise self.error('XPST0051', msg) from None
             else:
                 if position and (occurs is None or occurs == '?'):
                     return False
@@ -299,30 +302,27 @@ def evaluate_instance_expression(self, context=None):
 
 @method('treat')
 def evaluate_treat_expression(self, context=None):
-    if len(self) > 2:
-        occurs = self[2].symbol
-    else:
-        occurs = self[1].occurrence
-
+    occurs = self[1].occurrence
     position = None
     castable_expr = []
     if self[1].symbol == 'empty-sequence':
         for _ in self[0].select(context):
-            raise self.wrong_sequence_type()
+            raise self.error('XPDY0050')
     elif self[1].label in ('kind test', 'sequence type', 'function test'):
         for position, item in enumerate(self[0].select(context)):
             result = self[1].evaluate(context)
             if isinstance(result, list) and not result:
-                raise self.wrong_sequence_type()
+                raise self.error('XPDY0050')
             elif position and (occurs is None or occurs == '?'):
-                raise self.wrong_sequence_type("more than one item in sequence")
+                raise self.error('XPDY0050', "more than one item in sequence")
             castable_expr.append(item)
         else:
             if position is None and occurs not in ('*', '?'):
-                raise self.wrong_sequence_type("the sequence cannot be empty")
+                raise self.error('XPDY0050', "the sequence cannot be empty")
     else:
+        type_name = self[1].source.rstrip('*+?')
         try:
-            qname = get_expanded_name(self[1].source, self.parser.namespaces)
+            qname = get_expanded_name(type_name, self.parser.namespaces)
         except KeyError as err:
             raise self.error('XPST0081', 'prefix {} not found'.format(str(err)))
 
@@ -331,19 +331,19 @@ def evaluate_treat_expression(self, context=None):
 
         for position, item in enumerate(self[0].select(context)):
             try:
-                if not self.parser.is_instance(item, qname):
-                    msg = f"item {item!r} is not of type {self[1].source!r}"
+                if not is_instance(item, qname, self.parser):
+                    msg = f"item {item!r} is not of type {type_name!r}"
                     raise self.error('XPDY0050', msg)
             except KeyError:
-                msg = "atomic type %r not found in in-scope schema types"
-                raise self.error('XPST0051', msg % self[1].source) from None
+                msg = f"atomic type {type_name!r} not found in in-scope schema types"
+                raise self.error('XPST0051', msg) from None
             else:
                 if position and (occurs is None or occurs == '?'):
-                    raise self.wrong_sequence_type("more than one item in sequence")
+                    raise self.error('XPDY0050', "more than one item in sequence")
                 castable_expr.append(item)
         else:
             if position is None and occurs not in ('*', '?'):
-                raise self.wrong_sequence_type("the sequence cannot be empty")
+                raise self.error('XPDY0050', "the sequence cannot be empty")
 
     return castable_expr
 
@@ -354,10 +354,10 @@ def evaluate_treat_expression(self, context=None):
 @method('cast', bp=63)
 def led_cast_expressions(self, left):
     self.parser.advance('as')
-    self.parser.expected_name('(name)', ':')
-    self[:] = left, self.parser.expression(rbp=self.rbp)
+    self.parser.expected_next('(name)', ':', 'Q{', message='an EQName expected')
+    self[:] = left, self.parser.expression(rbp=85)
     if self.parser.next_token.symbol == '?':
-        self[2:] = self.parser.symbol_table['?'](self.parser),  # Add nullary token
+        self[1].occurrence = '?'
         self.parser.advance()
     return self
 
@@ -365,8 +365,9 @@ def led_cast_expressions(self, left):
 @method('castable')
 @method('cast')
 def evaluate_cast_expressions(self, context=None):
+    type_name = self[1].source.rstrip('+*?')
     try:
-        atomic_type = get_expanded_name(self[1].source, namespaces=self.parser.namespaces)
+        atomic_type = get_expanded_name(type_name, namespaces=self.parser.namespaces)
     except KeyError as err:
         raise self.error('XPST0081', 'prefix {} not found'.format(str(err)))
 
@@ -377,20 +378,20 @@ def evaluate_cast_expressions(self, context=None):
     if namespace != XSD_NAMESPACE and \
             (self.parser.schema is None or self.parser.schema.get_type(atomic_type) is None):
         msg = "atomic type %r not found in the in-scope schema types"
-        raise self.unknown_atomic_type(msg % atomic_type)
+        raise self.error('XPST0051', msg % atomic_type)
 
     result = [res for res in self[0].select(context)]
     if len(result) > 1:
         if self.symbol != 'cast':
             return False
-        raise self.wrong_context_type("more than one value in expression")
+        raise self.error('XPTY0004', "more than one value in expression")
     elif not result:
-        if len(self) == 3:
+        if self[1].occurrence == '?':
             return [] if self.symbol == 'cast' else True
         elif self.symbol != 'cast':
             return False
         else:
-            raise self.wrong_context_type("an atomic value is required")
+            raise self.error('XPTY0004', "an atomic value is required")
 
     arg = self.data_value(result[0])
     try:
@@ -400,8 +401,8 @@ def evaluate_cast_expressions(self, context=None):
             local_name = atomic_type.split('}')[1]
             token_class = self.parser.symbol_table.get(local_name)
             if token_class is None or token_class.label != 'constructor function':
-                msg = "atomic type %r not found in the in-scope schema types"
-                raise self.unknown_atomic_type(msg % self[1].source)
+                msg = f"atomic type {type_name!r} not found in the in-scope schema types"
+                raise self.error('XPST0051', msg)
             elif local_name == 'QName':
                 if isinstance(arg, QName):
                     pass
@@ -517,7 +518,7 @@ def evaluate_value_comparison_operators(self, context=None):
                 self[1].get_atomized_operand(context=copy(context))]
 
     if any(x is None for x in operands):
-        return None
+        return []
     elif any(isinstance(x, XPathFunction) for x in operands):
         raise self.error('FOTY0013', "cannot compare a function item")
     elif all(isinstance(x, DoubleProxy10) for x in operands):
@@ -541,11 +542,13 @@ def evaluate_value_comparison_operators(self, context=None):
         pass
     elif all(isinstance(x, (str, UntypedAtomic, AnyURI)) for x in operands):
         pass
+    elif all(isinstance(x, (str, UntypedAtomic, QName)) for x in operands):
+        pass
     elif all(isinstance(x, (float, Decimal, int)) for x in operands):
         if isinstance(operands[0], float):
-            operands[1] = float(operands[1])
+            operands[1] = get_double(operands[1], self.parser.xsd_version)
         else:
-            operands[0] = float(operands[0])
+            operands[0] = get_double(operands[0], self.parser.xsd_version)
     elif all(isinstance(x, Duration) for x in operands) and self.symbol in ('eq', 'ne'):
         pass
     elif (issubclass(cls0, cls1) or issubclass(cls1, cls0)) and not issubclass(cls0, Duration):
@@ -578,35 +581,30 @@ def evaluate_node_comparison(self, context=None):
 
     left = [x for x in self[0].select(context)]
     if not left:
-        return
-    elif len(left) > 1 or not is_xpath_node(left[0]):
+        return []
+    elif len(left) > 1 or not isinstance(left[0], XPathNode):
         raise self[0].error('XPTY0004', "left operand of %r must be a single node" % symbol)
 
     right = [x for x in self[1].select(context)]
     if not right:
-        return
-    elif len(right) > 1 or not is_xpath_node(right[0]):
+        return []
+    elif len(right) > 1 or not isinstance(right[0], XPathNode):
         raise self[0].error('XPTY0004', "right operand of %r must be a single node" % symbol)
 
-    # For identity comparison use '==' operator instead of 'is'
-    # because elem1 == elem2 if and only if elem1 is elem2.
-    # For example two AttributeNode objects (a1, a2) represent
-    # the same node if they have the same name, the same value
-    # and the same parent.
     if symbol == 'is':
-        return left[0] == right[0]
+        return left[0] is right[0]
     else:
-        if left[0] == right[0]:
+        if left[0] is right[0]:
             return False
 
         documents = [context.root]
-        documents.extend(v for v in context.variables.values() if is_document_node(v))
+        documents.extend(v for v in context.variables.values() if isinstance(v, DocumentNode))
 
         for root in documents:
-            for item in root.iter():  # pragma: no cover
-                if left[0] == item:
+            for item in root.iter_document():  # pragma: no cover
+                if left[0] is item:
                     return True if symbol == '<<' else False
-                elif right[0] == item:
+                elif right[0] is item:
                     return False if symbol == '<<' else True
         else:
             raise self.error('FOCA0002', "operands are not nodes of the XML tree!")
@@ -682,8 +680,7 @@ def evaluate_idiv_operator(self, context=None):
 @method('castable')
 @method('cast')
 def nud_disambiguation_of_infix_operators(self):
-    token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
-    return token.nud()
+    return self.as_name()
 
 
 ###
@@ -693,15 +690,16 @@ def select_document_node_kind_test(self, context=None):
     if context is None:
         raise self.missing_context()
     elif not self:
-        if is_document_node(context.item):
+        if isinstance(context.item, DocumentNode):
             yield context.item
-        elif is_document_node(context.root) and context.item is None:
+        elif isinstance(context.root, DocumentNode) and context.item is None:
             for item in context.iter_children_or_self():
                 if item is None:
                     yield context.root
     else:
-        elements = [e for e in self[0].select(copy(context)) if is_element_node(e)]
-        if is_document_node(context.root) and context.item is None:
+        elements = [e for e in self[0].select(copy(context)) if isinstance(e, ElementNode)]
+        if isinstance(context.root, DocumentNode) and context.item is None \
+                or isinstance(context.item, DocumentNode):
             if len(elements) == 1:
                 yield context.root
 
@@ -712,7 +710,8 @@ def nud_document_node_kind_test(self):
     if self.parser.next_token.symbol in ('element', 'schema-element'):
         self[0:] = self.parser.expression(5),
         if self.parser.next_token.symbol == ',':
-            raise self.wrong_nargs('Too many arguments: expected at most 1 argument')
+            msg = 'Too many arguments: expected at most 1 argument'
+            raise self.error('XPST0017', msg)
     elif self.parser.next_token.symbol != ')':
         raise self.error('XPST0003', 'element or schema-element kind test expected')
     self.parser.advance(')')
@@ -726,21 +725,27 @@ def select_element_kind_test(self, context=None):
         raise self.missing_context()
     elif not self:
         for item in context.iter_children_or_self():
-            if is_element_node(item):
+            if isinstance(item, ElementNode):
                 yield item
     else:
         for item in self[0].select(context):
             if len(self) == 1:
                 yield item
-            elif isinstance(item, TypedElement):
+            elif isinstance(item, ElementNode):
                 try:
                     type_annotation = get_expanded_name(self[1].source, self.parser.namespaces)
                 except KeyError:
                     type_annotation = self[1].source
 
-                if type_annotation == item.xsd_type.name:
+                if item.nilled:
+                    if type_annotation[-1] in '*?':
+                        yield item
+                elif item.xsd_type is None:
+                    if type_annotation == XSD_UNTYPED and self[0].symbol != '*':
+                        yield item
+                elif type_annotation == item.xsd_type.name:
                     yield item
-                elif item.elem.get(XSI_NIL) and type_annotation[-1] in ('*', '?'):
+                elif is_instance(item.typed_value, type_annotation, self.parser):
                     yield item
 
 
@@ -748,12 +753,12 @@ def select_element_kind_test(self, context=None):
 def nud_element_kind_test(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol != ')':
-        self.parser.expected_name('(name)', ':', '*', message='a QName or a wildcard expected')
+        self.parser.expected_next('(name)', ':', '*', message='a QName or a wildcard expected')
         self[0:] = self.parser.expression(5),
         if self.parser.next_token.symbol == ',':
             self.parser.advance(',')
-            self.parser.expected_name('(name)', ':', message='a QName expected')
-            self[1:] = self.parser.expression(5),
+            self.parser.expected_next('(name)', ':', message='a QName expected')
+            self[1:] = self.parser.expression(80),
             if self.parser.next_token.symbol in ('*', '+', '?'):
                 self[1].occurrence = self.parser.next_token.symbol
                 self.parser.advance()
@@ -772,9 +777,9 @@ def select_schema_attribute_kind_test(self, context=None):
     for _ in context.iter_children_or_self():
         qname = get_expanded_name(attribute_name, self.parser.namespaces)
         if self.parser.schema.get_attribute(qname) is None:
-            raise self.missing_name("attribute %r not found in schema" % attribute_name)
+            raise self.error('XPST0008', "attribute %r not found in schema" % attribute_name)
 
-        if match_attribute_node(context.item, qname):
+        if isinstance(context.item, AttributeNode) and context.item.match_name(qname):
             yield context.item
             return
 
@@ -788,15 +793,17 @@ def select_schema_element_kind_test(self, context=None):
         raise self.missing_context()
 
     element_name = self[0].source
-    for _ in context.iter_children_or_self():
-        qname = get_expanded_name(element_name, self.parser.namespaces)
-        if self.parser.schema.get_element(qname) is None \
-                and self.parser.schema.get_substitution_group(qname) is None:
-            raise self.missing_name("element %r not found in schema" % element_name)
 
-        if is_element_node(context.item) and context.item.tag == qname:
-            yield context.item
-            return
+    if self.parser.schema is not None:
+        for _ in context.iter_children_or_self():
+            qname = get_expanded_name(element_name, self.parser.namespaces)
+            if self.parser.schema.get_element(qname) is None \
+                    and self.parser.schema.get_substitution_group(qname) is None:
+                raise self.error('XPST0008', "element %r not found in schema" % element_name)
+
+            if isinstance(context.item, ElementNode) and context.item.elem.tag == qname:
+                yield context.item
+                return
 
     if not isinstance(context, XPathSchemaContext):
         raise self.error('XPST0008', 'schema element %r not found' % element_name)
@@ -806,7 +813,7 @@ def select_schema_element_kind_test(self, context=None):
 @method('schema-element')
 def nud_schema_node_kind_test(self):
     self.parser.advance('(')
-    self.parser.expected_name('(name)', ':', message='a QName expected')
+    self.parser.expected_next('(name)', ':', 'Q{', message='a QName expected')
     self[0:] = self.parser.expression(5),
     self.parser.advance(')')
     self.value = None
@@ -833,7 +840,7 @@ def nud_attribute_kind_test_or_axis(self):
     if self.parser.next_token.symbol == '::':
         self.label = 'axis'
         self.parser.advance('::')
-        self.parser.expected_name(
+        self.parser.expected_next(
             '(name)', '*', 'text', 'node', 'document-node', 'comment', 'processing-instruction',
             'attribute', 'schema-attribute', 'element', 'schema-element', 'namespace-node'
         )
@@ -877,15 +884,24 @@ def select_attribute_kind_test_or_axis(self, context=None):
             type_name = None
 
         for attribute in context.iter_attributes():
-            if match_attribute_node(attribute, name):
+            if attribute.match_name(name):
                 if isinstance(context, XPathSchemaContext):
                     self.add_xsd_type(attribute)
-                elif not type_name:
-                    yield attribute.value
+                    continue
+
+                if attribute.xsd_type is None:
+                    attribute.xsd_type = self.get_xsd_type(attribute)
+
+                if not type_name:
+                    yield attribute.typed_value
                 else:
-                    xsd_type = self.get_xsd_type(attribute)
-                    if xsd_type is not None and xsd_type.name == type_name:
-                        yield attribute.value
+                    if attribute.xsd_type is None:
+                        if type_name == XSD_UNTYPED and name != '*':
+                            yield attribute.value
+                    elif attribute.xsd_type.name == type_name:
+                        yield attribute.typed_value
+                    elif is_instance(attribute.typed_value, type_name, self.parser):
+                        yield attribute.typed_value
 
 
 # XPath 2.0 definitions continue into module xpath2_functions
