@@ -17,7 +17,7 @@ from typing import TYPE_CHECKING, cast, overload, Dict, Any, List, Iterator, \
 
 from .exceptions import ElementPathTypeError
 from .namespaces import NamespacesType
-from .datatypes import AnyAtomicType, Timezone
+from .datatypes import AnyAtomicType, Timezone, Language
 from .protocols import ElementProtocol, DocumentProtocol
 from .etree import is_etree_element, is_etree_document
 from .xpath_nodes import RootArgType, ChildNodeType, XPathNode, \
@@ -26,7 +26,7 @@ from .xpath_nodes import RootArgType, ChildNodeType, XPathNode, \
 from .tree_builders import get_node_tree
 
 if TYPE_CHECKING:
-    from .xpath_token import XPathToken, XPathAxis
+    from .xpath_tokens import XPathToken, XPathAxis
 
 __all__ = ['XPathContext', 'XPathSchemaContext']
 
@@ -149,7 +149,7 @@ class XPathContext:
         self.resource_collections = resource_collections
         self.default_resource_collection = default_resource_collection
         self.allow_environment = allow_environment
-        self.default_language = default_language
+        self.default_language = None if default_language is None else Language(default_language)
         self.default_calendar = default_calendar
         self.default_place = default_place
 
@@ -356,15 +356,16 @@ class XPathContext:
         if not isinstance(self.item, XPathNode):
             return  # not applicable
 
-        parent = self.item.parent
-        if parent is not None:
-            status = self.item, self.axis
-            self.axis = 'parent'
+        if self.item is not self.root:
+            parent = self.item.parent
+            if parent is not None:
+                status = self.item, self.axis
+                self.axis = 'parent'
 
-            self.item = parent
-            yield self.item
+                self.item = parent
+                yield self.item
 
-            self.item, self.axis = status
+                self.item, self.axis = status
 
     def iter_siblings(self, axis: Optional[str] = None) -> Iterator[ChildNodeType]:
         """
@@ -372,7 +373,7 @@ class XPathContext:
 
         :param axis: the context axis, default is 'following-sibling'.
         """
-        if not isinstance(self.item, XPathNode):
+        if not isinstance(self.item, XPathNode) or self.item is self.root:
             return
 
         parent = self.item.parent
@@ -448,10 +449,13 @@ class XPathContext:
         if axis == 'ancestor-or-self':
             ancestors.append(self.item)
 
-        parent = self.item.parent
-        while parent is not None:
-            ancestors.append(parent)
-            parent = parent.parent
+        if self.item is not self.root:
+            parent = self.item.parent
+            while parent is not None:
+                ancestors.append(parent)
+                if parent is self.root:
+                    break
+                parent = parent.parent
 
         for self.item in reversed(ancestors):
             yield self.item
@@ -464,7 +468,7 @@ class XPathContext:
         item: XPathNode
         parent: Union[None, ElementNode, DocumentNode]
 
-        if not isinstance(self.item, XPathNode):
+        if not isinstance(self.item, XPathNode) or self.item is self.root:
             return
 
         parent = self.item.parent
@@ -477,6 +481,8 @@ class XPathContext:
         ancestors = set()
         while parent is not None:
             ancestors.add(parent)
+            if parent is self.root:
+                break
             parent = parent.parent
 
         item = self.item
