@@ -1311,9 +1311,7 @@ class QNameTypesTest(unittest.TestCase):
         self.assertNotEqual(qname1, qname3)
         self.assertNotEqual(qname2, qname3)
 
-        with self.assertRaises(TypeError) as ctx:
-            _ = qname1 == 'tst1:foo'
-        self.assertIn('cannot compare', str(ctx.exception))
+        self.assertEqual(qname1, 'tst1:foo')
 
         with self.assertRaises(TypeError) as ctx:
             _ = qname1 == 1
